package ru.dbotthepony.kstarbound.client.gl.vertex

import org.lwjgl.opengl.GL46
import ru.dbotthepony.kstarbound.PIXELS_IN_STARBOUND_UNITf
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.GLTexture2D
import ru.dbotthepony.kstarbound.client.gl.checkForGLError
import java.io.Closeable

/**
 * Класс-помощник для быстрого построения, загрузки и отрисовки геометрии
 */
class StreamVertexBuilder(
	val state: GLStateTracker,
	attributes: GLAttributeList,
	type: GeometryType,
	maxElements: Int,
) : DirectVertexBuilder<StreamVertexBuilder>(attributes, type, maxElements), Closeable {
	private val vao = state.newVAO()
	private val vbo = state.newVBO()
	private val ebo = state.newEBO()

	init {
		vao.bind()
		vbo.bind()
		ebo.bind()

		attributes.apply(vao, true)

		vao.unbind()
		vbo.unbind()
		ebo.unbind()
	}

	fun upload(drawType: Int = GL46.GL_DYNAMIC_DRAW) {
		upload(vbo, ebo, drawType)
	}

	fun bind() = vao.bind()
	fun unbind() = vao.unbind()

	fun draw(primitives: Int = GL46.GL_TRIANGLES) {
		bind()
		GL46.glDrawElements(primitives, indexCount, elementIndexType, 0L)
		checkForGLError()
	}

	override fun close() {
		vao.close()
		vbo.close()
		ebo.close()
	}

	fun singleSprite(x: Float, y: Float, width: Float, height: Float, z: Float = 5f, angle: Double = 0.0, transformer: QuadVertexTransformer) {
		begin()

		quadRotatedZ(x, y, width, height, z, 0f, 0f, angle, transformer)

		upload()
		draw()
	}

	fun singleSprite(width: Float, height: Float, z: Float = 5f, angle: Double = 0.0, transformer: QuadVertexTransformer) {
		singleSprite(-width / 2f, -height / 2f, width / 2f, height / 2f, z, angle, transformer)
	}

	fun singleSprite(width: Float, height: Float, angle: Double = 0.0, transformer: QuadVertexTransformer) {
		singleSprite(-width / 2f, -height / 2f, width / 2f, height / 2f, 0f, angle, transformer)
	}
}
